import perl
#if perl.MULTI_PERL:
#	print "1..0"
#	raise SystemExit

print("1..16")

a = perl.get_ref("@")

if a.__type__ != 'ARRAY' or a.__class__ != None: print("not", end=' ')
print("ok 1")

try:
	print(a["foo"])
except TypeError as v:
	if str(v) != "perl array index must be integer": print("not", end=' ')
	print("ok 2")

try:
	print(a[10])
except IndexError as v:
        if str(v) != "perl array index out of range": print("not", end=' ')
        print("ok 3")

try:
	a[4] = 5
except IndexError as v:
	if str(v) != "perl array assignment index out of range": print("not", end=' ')
	print("ok 4")

a.extend(list(range(5)))

a_copy = a[:]

if list(a) != list(range(5)) or list(a) != list(a_copy): print("not", end=' ')
print("ok 5")

a.append(2)

if a.index(2) != 2 or a.count(2) != 2 or a.count(99) != 0: print("not", end=' ')
print("ok 6")

if a.pop() != 2 or len(a) != 5: print("not", end=' ')
print("ok 7")

a.insert(0, "foo")
if a.pop(0) != "foo" or len(a) != 5: print("not", end=' ')
print("ok 8")

a.append(3)
a.remove(3)
if len(a) != 5 or a.index(3) != 4: print("not", end=' ')
print("ok 9")

a.remove(3)
try:
	a.remove(3)
except ValueError as v:
	if str(v) != "perlarray.remove(x): x not in list": print("not", end=' ')
	print("ok 10")

try:
	print(a.index(3))
except ValueError as v:
	if str(v) != "perlarray.index(x): x not in list": print("not", end=' ')
	print("ok 11")

# restore list
a[3:3] = a_copy[3:4]

a.reverse()

if list(a) != list(range(4, -1, -1)): print("not", end=' ')
print("ok 12")

a.reverse()
if list(a) != list(range(5)): print("not", end=' ')
print("ok 13")

a.extend("abc")

if list(a) != list(range(5)) + list("abc"): print("not", end=' ')
print("ok 14")

a.extend(a)
if list(a) != (list(range(5)) + list("abc"))*2: print("not", end=' ')
print("ok 15")

a[5:] = perl.array([]);
if list(a) != list(range(5)): print("not", end=' ')
print("ok 16")



#!/usr/bin/env python

import unittest
import perl

class TestPyPerlBasic(unittest.TestCase):

    def setUp(self):

        perl.eval("""
sub foo2 {
    wantarray ? (1, 2, 3) : 42;
}
""")

    def test_simple_calculator(self):
        # try to use perl as a simple calculator
        self.assertEqual(perl.eval("3+3"), 6)

    def test_pass_strings_back(self):
        # can we pass strings back
        self.assertEqual(perl.eval("substr('abcd', 0, 3)"), "abc")

    def test_pass_hashes_both_ways(self):
        # can we pass hashes both ways
        if perl.MULTI_PERL:
            self.skipTest("not on MULTI_PERL...")
        else:
            perl.eval("sub foo_elem { shift->{foo} }")
            hash = perl.eval("{ foo => 42 }")
            self.assertEqual(perl.call("foo_elem", hash), 42)

    def test_trap_exceptions(self):
        # try to trap exceptions
        try:
            perl.eval("die 'Oops!'")
        except perl.PerlError as val:
            self.assertEqual(str(val)[:5],"Oops!")

        try:
            perl.call("not_there", 3, 4)
        except perl.PerlError as val:
            self.assertEqual(str(val), "Undefined subroutine &main::not_there called.\n")


    def test_function_scalar_context(self):
        # scalar context
        self.assertEqual(perl.call("foo2"), 42)

    def test_function_array_context_tuple_back(self):
        res = perl.call_tuple("foo2")
        self.assertEqual(len(res), 3)
        self.assertEqual(res[0], 1)
        self.assertEqual(res[1], 2)
        self.assertEqual(res[2], 3)

    def test_anonymous_perl_functions(self):
        # can we call anonymous perl functions
        # can we pass hashes both ways
        if perl.MULTI_PERL:
            self.skipTest("not on MULTI_PERL...")
        else:
            func = perl.eval("sub { $_[0] + $_[1] }")
            self.assertEqual(int(func(3, 4)), 7)

def test_main():
    from test import test_support
    test_support.run_unittest(TestPyPerlBasic)

if __name__ == "__main__":
    test_main()



import perl
if perl.MULTI_PERL:
	print("1..0")
	raise SystemExit


try:
	import _thread
except:
	print("1..0")
	print("Threads not supported by python")
	raise SystemExit

print("1..1")
import time

inc = perl.get_ref("@INC")
i0 = inc[0]

work_started = 0
work_done = 0

def wait():
    global work_started, work_done
    while not work_started:
        time.sleep(0.1)
    work_started = 0
    while not work_done:
        time.sleep(0.1)

def start():
    global work_started
    work_started = 1
    
def done():
    global work_done
    work_done = 1

def t1():
    start()
    global inc, i0
    if i0 != inc[0]: print("not", end=' ')
    print("ok 1");
    done()

_thread.start_new_thread(t1, ())
wait()



print("1..8")

import perl

# try to use perl as a simple calculator
if not perl.eval("3+3") == 6: print("not", end=' ')
print("ok 1")

# can we pass strings back
if not perl.eval("substr('abcd', 0, 3)") == "abc": print("not", end=' ')
print("ok 2")

# can we pass hashes both ways
if perl.MULTI_PERL:
    print("actually skipping test 3...")
    print("ok 3")
else:
    perl.eval("sub foo_elem { shift->{foo} }")
    hash = perl.eval("{ foo => 42 }")

    if not perl.call("foo_elem", hash) == 42: print("not", end=' ')
    print("ok 3")

# try to trap exceptions
try:
	perl.eval("die 'Oops!'")
except perl.PerlError as val:
	if str(val)[:5] != "Oops!": print("not", end=' ')
	print("ok 4") 

try:
	perl.call("not_there", 3, 4)
except perl.PerlError as val:
	if str(val) != "Undefined subroutine &main::not_there called.\n":
		print("not", end=' ')
	print("ok 5")


# try calling perl function in array and scalar context

# first define a function
perl.eval("""
   sub foo2 {
	wantarray ? (1, 2, 3) : 42;
   }
""")

# scalar context
if perl.call("foo2") != 42: print("not", end=' ')
print("ok 6")

# array context (tuple back)
res = perl.call_tuple("foo2")
if len(res) != 3 or res[0] != 1 or res[1] != 2 or res[2] != 3: print("not", end=' ')
print("ok 7")


# can we call anonymous perl functions
# can we pass hashes both ways
if perl.MULTI_PERL:
    print("actually skipping test 8...")
    print("ok 8")
else:
    func = perl.eval("sub { $_[0] + $_[1] }")
    if int(func(3, 4)) != 7: print("not", end=' ')
    print("ok 8")



import perl
#if (perl.MULTI_PERL):
#    print "1..0"
#    raise SystemExit

print("1..14")

def ok(a, b=None):
    return "a=" + str(a) + ", b=" + str(b)

perl.eval("""

use Python qw(apply);

$| = 1;

sub {
    my $f = shift;

    # First some tests that are expected to blow up
    eval {
       apply($f);
    };
    #print $@;

    # XXX For some strange reason =~ is not to force $@ to stingify, so
    # I had to help it with "$@" =~.
    # Hmmm, something to fix some other time :-(
    print "not " unless "$@" =~ /^python\.<type 'exceptions.TypeError'>: ok\(\) takes at least 1 argument \(0 given\)/;
    print "ok 1\n";

    eval {
       apply($f, undef);
    };
    #print $@;
    print "not " unless "$@" =~ /^python\.<type 'exceptions.TypeError'>: ok\(\) takes at least 1 argument \(0 given\)/;
    print "ok 2\n";

    eval {
       apply($f, undef, undef);
    };
    #print $@;
    print "not " unless "$@" =~ /^python\.<type 'exceptions.TypeError'>: ok\(\) takes at least 1 argument \(0 given\)/;
    print "ok 3\n";

    eval {
       apply($f, undef, undef, undef);
    };
    #print $@;
    print "not " unless "$@" =~ /^Too many arguments at \(eval 1\) line \d+./;
    print "ok 4\n";

    eval {
       apply($f, [1,2,3]);
    };
    #print $@;
    print "not " unless "$@" =~ /^python\.<type 'exceptions.TypeError'>: ok\(\) takes at most 2 arguments \(3 given\)/;
    print "ok 5\n";

    eval {
       apply($f, [], {b => 2});
    };
    #print $@;
    print "not " unless "$@" =~ /^python\.<type 'exceptions.TypeError'>: ok\(\) takes at least 1 non-keyword argument \(0 given\)/;
    print "ok 6\n";

    eval {
       apply($f, [1], {a => 2});
    };
    #print $@;
    print "not " unless "$@" =~ /^python\.<type 'exceptions.TypeError'>: ok\(\) got multiple values for keyword argument 'a'/;
    print "ok 7\n";

    eval {
       apply($f, [], {a => 2, b => 3, c => 4});
    };
    #print $@;
    print "not " unless "$@" =~ /^python\.<type 'exceptions.TypeError'>: ok\(\) got an unexpected keyword argument 'c'/;
    print "ok 8\n";

    eval {
        apply($f, 1);
    };
    #print $@;
    print "not " unless "$@" =~ /^/;
    print "ok 9\n";

    # Then some tests that are expected to work
    $res = apply($f, undef, { a => 101, b => 102 });
    #print "$res\\n";
    print "not " unless $res eq "a=101, b=102";
    print "ok 10\n";

    $res = apply($f, undef, { a => 101 });
    #print "$res\\n";
    print "not " unless $res eq "a=101, b=None";
    print "ok 11\n";

    $res = apply($f, [101, 102]);
    #print "$res\\n";
    print "not " unless $res eq "a=101, b=102";
    print "ok 12\n";

    $res = apply($f, Python::list(101, 102), Python::dict());
    #print "$res\\n";
    print "not " unless $res eq "a=101, b=102";
    print "ok 13\n";

    $res = apply($f, [], Python::dict(a => 101));
    #print "$res\\n";
    print "not " unless $res eq "a=101, b=None";
    print "ok 14\n";

}


""")(ok)



#!/usr/bin/env python

import unittest
import perl

class TestPyPerlEval(unittest.TestCase):

    def test_eval(self):
        self.assertEqual(perl.eval("""
Python::exec("print('ok 1'); n = 4");
Python::eval("n/2");
"""), 2.0);


def test_main():
    from test import test_support
    test_support.run_unittest(TestPyPerlEval)

if __name__ == "__main__":
    test_main()



#!/usr/bin/env python

import unittest
import perl

class TestPyPerlDefined(unittest.TestCase):

    def setUp(self):
        perl.eval(
                """sub foo;"""
                """sub bar { }"""
                """@baz = ();"""
                """$Foo::bar = 33;""")

    def test_defined(self):
        self.assertFalse(perl.defined("baz"))

        self.assertTrue(perl.defined("foo"))
        self.assertTrue(perl.defined("bar"))

        self.assertTrue(perl.defined("@baz"))

        self.assertTrue(perl.defined("$Foo::bar"))

        self.assertTrue(perl.defined("$Foo::bar"))

def test_main():
    from test import test_support
    test_support.run_unittest(TestPyPerlDefined)

if __name__ == "__main__":
    test_main()

# vim:ts=4:sw=4:et



# After this module is loaded, perl object can be pickled
# The perl objects can even contain python objects that contain
# perl objects that contain python objects that...
#
# You are not supposed to use any functions from this module.
# Just use Pickle as usual.
#
# Copyright 2000-2001 ActiveState

import perl
perl.require("Storable")
perl.callm("VERSION", "Storable", 0.7);

storable_thaw = perl.get_ref("Storable::thaw")
storable_nfreeze = perl.get_ref("Storable::nfreeze")

def perl_restore(frozen):
    return storable_thaw(frozen)

def perl_reduce(o):
    return (perl_restore,
            (storable_nfreeze(o),)
           )

import copyreg
copyreg.pickle(type(perl.get_ref("$")), perl_reduce, perl_restore)
del(copy_reg)

from pickle import dumps, loads

# Make the dumps and loads functions available for perl
f = perl.get_ref("$Python::Object::pickle_dumps", 1)
f.__value__ = dumps;
f = perl.get_ref("$Python::Object::pickle_loads", 1)
f.__value__ = loads;
del(f)

perl.eval("""

package Python::Object;

sub STORABLE_freeze {
   my($self, $cloning) = @_;
   return Python::funcall($pickle_dumps, $self, 1);
}

sub STORABLE_thaw {
   my($self, $cloning, $serialized) = @_;
   my $other = Python::funcall($pickle_loads, $serialized);
   Python::PyO_transplant($self, $other);
   return;
}

""")



import perl
#if perl.MULTI_PERL:
#	print "1..0"
#	raise SystemExit

print("1..6")

perl.eval("use lib 't'")     # good when running from ..
perl.eval("use TestClass")

obj = perl.callm("new", "TestClass")

#print obj
#print type(obj)
#print repr(obj)

# Test plain method calls

if obj.foo(42) != None: print("not", end=' ')
print("ok 1")

if obj.foo() != 42: print("not", end=' ')
print("ok 2")

my_dict = {}
obj.foo(my_dict)

if obj.foo() is not my_dict: print("not", end=' ')
print("ok 3")

obj.foo(obj.newhash("key", 42))
try:
	obj.dump()
except perl.PerlError as v:
	print(v)

if int(obj.hash_deref(obj.foo(), "key")) != 42: print("not", end=' ')
print("ok 4")

# calling in scalar/array context
#print obj.localtime()
#print obj.localtime_tuple()
#print "----"

# callin back to python
class Foo:
	def foo(self, a):
		#print "method foo called with argument", a
		return 12/a
	pass

p_obj = Foo()
x = obj.callback(p_obj, "foo", 3)
if x != 4: print("not", end=' ')
print("ok 5")

try:
   obj.callback(p_obj, "foo", 0)
except ZeroDivisionError as v:
   if str(v) != "integer division or modulo by zero": print("not", end=' ')
   print("ok 6")





import perl
import sys

print("1..4")

perl.eval("use lib 't'")     # good when running from ..
perl.eval("use TestClass")

try:
	obj = perl.callm("new", "NotAClass")
except perl.PerlError as v:
	expect = "Can't locate object method \"new\" via package \"NotAClass\""
	if str(v)[:len(expect)] != expect:
		print("not", end=' ')
	print("ok 1")

obj = perl.callm("new", "TestClass")

try:
	obj.not_a_method(34, 33);
except perl.PerlError as v:
	if str(v) != "Can't locate object method \"not_a_method\" via package \"TestClass\".\n":
		print("not", end=' ')
	print("ok 2")
except AttributeError:
	if not perl.MULTI_PERL:
		print("not", end=' ')
	print("ok 2")

try:
	obj.error("foo");
except perl.PerlError as v:
	if str(v)[:15] != "Failed: foo at ": print("not", end=' ')
	print("ok 3")
except AttributeError:
	if not perl.MULTI_PERL:
		print("not", end=' ')
	print("ok 3")

try:
	perl.eval("{}").xyzzy()
except AttributeError as v:
	from string import find
	if find(str(v), "xyzzy") == -1:
		#print v
		print("not", end=' ')
	print("ok 4")



# Copyright 2001 ActiveState

"""perlmod - Simplified access to Perl modules

This module provide adaptors that makes it easier to access and use
Perl modules from Python.  It provide classes to encapsulate Perl
modules and Perl classes.

Example usage of an Perl OO module:

   from perlmod import Perl
   HTTP = Perl.HTTP

   ua = Perl.LWP.UserAgent()
   res = ua.request(HTTP.Request("GET", "http://www.python.org"))
   if res.is_success():
       print res.content()
   else:
       print res.status_line()

Another way to do it:

   from perlmod import PerlClass

   # Import classes
   LWP_UserAgent = PerlClass("LWP::UserAgent")
   HTTP_Request  = PerlClass("HTTP::Request")

   # do the stuff
   ua = LWP_UserAgent()
   res = ua.request(HTTP_Request("GET", "http://www.python.org"))
   if res.is_success():
       print res.content()
   else:
       print res.status_line()

An attribute prefixed with '__' is regarded as a class methods:

   from perlmod import Perl
   u = Perl.URI.file.__cwd()  # $u = URI::file->cwd;
   print u

Example usage of an functional style module:

   from perlmod import PerlModule
   print PerlModule("MIME::Base64").encode("foo")

Explicit manual import:

   from perlmod import PerlModule
   encode_base64 = PerlModule("MIME::Base64").encode

   print encode_base64("foo")

Import all functions that are exported by default (@EXPORT):

   from perlmod import PerlModule
   PerlModule("MIME::Base64").__import__("*", locals())   

   print encode_base64("foo")
"""

class PerlClass:
    def __init__(self, name = None, module=None, ctor="new"):
        self.name = name
        self.module = module or name
        self.ctor = ctor
        
    def __getattr__(self, name):
        if name[:2] == '__':
            if name[-2:] != '__' and name != '__':
                return PerlClass(self.name, ctor=name[2:])
            raise AttributeError(name)
        if self.name:
            name = self.name + "::" + name
        return PerlClass(name)

    def __call__(self, *args):
        import perl
        name = self.name
        perl_require(self.module)
        return perl.callm(*(self.ctor, name) + args)

class PerlModule:
    def __init__(self, name, __wantarray__ = 0):
        self.name = name
        self.wantarray = __wantarray__

    def __getattr__(self, name):
        if name[:2] == '__':
                raise AttributeError(name)
        perl_require(self.name)
        wantarray = self.wantarray
        if len(name) > 6 and name[-6:] == '_tuple':
            name = name[:-6]
            wantarray = 1
        full_name = self.name + "::" + name
        import perl
        func = perl.get_ref(full_name)
        func.__wantarray__ = wantarray
        return func

    def __import__(self, funcs, namespace):
        perl_require(self.name)
        import perl
        if funcs == '*':
            funcs = tuple(perl.get_ref("@" + self.name + "::EXPORT"))
        elif type(funcs) == type(""):
            funcs = (funcs,)
        for f in funcs:
            namespace[f] = perl.get_ref(self.name + "::" + f)

Perl = PerlClass()


# Loading of perl modules

INC = {}

try:
    from _thread import get_ident
except ImportError:
    def get_ident():
        return 1

def perl_require(mod):
    # Some caching since the real 'perl.require' is a bit
    # heavy.
    id = get_ident()
    global INC
    try:
        return INC[id][mod]
    except KeyError:
        pass
    
    import perl
    if id not in INC:
        INC[id] = {}
    INC[id][mod] = perl.require(mod)
    return INC[id][mod]




# A basic thread test

try:
    import _thread
except:
    print("1..0")
    import sys
    sys.exit(0)

print("1..1")
import time
import perl

perl.eval("""
$| = 1;
$a = "0";
""")

t1_done = 0
t2_done = 0

def t1():
    global t1_done
    time.sleep(0.2)
#    perl.eval("""print "Hello 1\n";""")
    perl.eval("""$a = "1a";""")
    time.sleep(0.7)
#    perl.eval("""print "Hello 1 again\n";""")
    perl.eval("""$a = "1b";""")
    t1_done = 1

def t2():
    global t2_done
    time.sleep(0.75)
#    perl.eval("""print "Hello 2\n";""")
    perl.eval("""$a = "2";""")
    t2_done = 1 #perl.eval("time")
    

_thread.start_new_thread(t1, ())
_thread.start_new_thread(t2, ())

# Main thread just waits for both children to terminate
count = 0
while not (t1_done and t2_done):
    time.sleep(0.1)
    count = count + 1
    if count > 100:
        break
#    print '.',

if count > 100: 
     print("not ", end=' ')
elif t1_done != 1 or t2_done != 1:
     print("not ", end=' ')
print("ok 1")




print("1..14")

class Foo:
    plain = 34
    def list(self):
        return [1,2,3]

    

foo = Foo()
foo.plain_list = [3, 2, 1]

import perl
perl.eval("""
# Test access to object from perl

use Python qw(getattr hasattr setattr getitem);

sub foo {
   my $foo = shift;

   print "not " unless hasattr($foo, "plain") && getattr($foo, "plain") == 34;
   print "ok 1\n";
   setattr($foo, plain => 42);
   print "not " unless getattr($foo,  "plain") eq "42";
   print "ok 2\n";
   Python::delattr($foo, "plain");
   print "not " unless Python::getattr($foo, "plain") eq "34";
   print "ok 3\n";

   print "not " unless $foo->plain eq 34;
   print "ok 4\n";

   $foo->plain2(72);
   print "not " unless getattr($foo, "plain2") eq 72;
   print "ok 5\n";

   print "not " unless $foo->plain2 eq 72;
   print "ok 6\n";

   print "not " unless $foo->plain2("bar") eq 72 && $foo->plain2 eq "bar";
   print "ok 7\n";

   my $list = $foo->plain_list;
   print "not " unless Python::len($list) == 3 && getitem($list, 0) == 3;
   print "ok 8\n";

   $list->append(0);
   print "not " unless Python::len($list) == 4 && $list->[-1] == 0;
   print "ok 9\n";

   my @list = $foo->plain_list;
   print "not " unless "@list" eq "3 2 1 0";
   print "ok 10\n";

   # try method call
   $list = $foo->list;
   @list = $foo->list;
   print "not " unless "$list" eq "[1, 2, 3]" && "@list" eq "1 2 3";
   print "ok 11\n";

   # try access to non-existing attribute
   eval {
       $foo->not_there;
   };
   print "not " unless $@ && Python::Err::AttributeError($@->type);
   print "ok 12\n";

   # try calling something which is not callable
   eval {
       $foo->plain_list("foo", "bar");
   };
   print "not " unless $@ && $@ =~ /^python\.<type 'exceptions\.TypeError'>: 'list' object is not callable/;
   print "ok 13\n";

   # Strings are a sequences too, but they are not unwrapped.
   $foo->string("string");
   @list = $foo->string;
   print "not " if "@list" ne "string";
   print "ok 14\n";
}

""")

perl.call("foo", foo)






#!//usr/bin/env python

import os
import sys

if sys.platform == 'win32':
	print((os.path.join(sys.prefix, 'include')))
else:
	print((os.path.join(sys.prefix, 'include', 'python' + sys.version[:3])))



import sys

try:
    import _thread
except:
    print("1..0")
    sys.exit(0)

import perl
if not perl.MULTI_PERL:
    print("1..0")
    sys.exit(0)

# This tests behaviour of perl objects passed from one
# thread (and perl interpreter) to the next one and that
# it is still destructed properly.

print("1..5")

perl_obj = perl.eval("""

sub Foo::hello {
   return "Hello";
}

sub Foo::DESTROY
{
   my $self = shift;
   print "ok 2\n";
}

bless {}, "Foo";

""")

#print perl_obj.hello();
#print perl_obj

def t1():
    global perl_obj
    try:
        perl_obj.hello()
        print("not ")
    except ValueError as v:
        print("ok 1")
        #print v

    perl.eval("""sub Foo::DESTROY { $|=1; print "ok 4\n"; }""");

    perl_obj = perl.get_ref("@")
    perl_obj.__class__ = "Foo";
    #print perl_obj
    print("ok 3")
    sys.stdout.flush();

_thread.start_new_thread(t1, ())

import time
time.sleep(2)
#print perl_obj
perl_obj = None

print("ok 5")






# This test that the embedded perl can load XS modules that happen
# to be (usually) separate dynamic libraries.

print("1..2")

import perl

addr = perl.eval("""
                 require Socket;
                 Socket::pack_sockaddr_in(80, Socket::inet_aton('127.0.0.1'));
                 """);

#print(repr(addr));

if not addr: print("not ", end=' ')
print("ok 1");

addr = perl.call_tuple("Socket::unpack_sockaddr_in", addr)
#print(repr(addr))

if addr[0] != 80 or len(addr[1]) != 4: print("not ", end=' ')
print("ok 2")




import perl

perl.eval("""

use Term::ReadLine;
my $term = Term::ReadLine->new("perlsh");


use Python @Python::EXPORT_OK;

sub shell {
   my $prompt = shift || "perlsh> ";
   while (defined($_ = $term->readline($prompt))) {
      chomp;
      my $res = eval $_;
      print "$res\n" if defined $res;
      print $@ if $@;
   }
   print "\n";
}

""")

perl.call("shell")





#!/usr/bin/env python

import dbi

for d in dbi.available_drivers():
	print(d);
	try:
		for s in dbi.data_sources(d):
			print("  ", s)
	except:
		pass
print("----")

#$dbh = DBI->connect("DBI:mysql:database=test", "snake", "",
#                    {
#                      RaiseError => 1,
#                      PrintError => 0,
#                      AutoCommit => 1,
#                    }) || die;


dbh = dbi.connect("DBI:mysql:database=test", "snake",
		  RaiseError = 1,
		  PrintError = 0,
		  AutoCommit = 1,
	         )

try:
    dbh["AutoCommit"] = 0
except:
    print("Can't turn off AutoCommit")

sth = dbh.prepare("select * from img limit 5")
rows = sth.execute()
print(rows)

while 1:
	row = sth.fetchrow_tuple()
        if not row: break
	print(row)

dbh.disconnect()



# Verify that perl executes concurrently as it should under MULTI_PERL

import perl
if not perl.MULTI_PERL:
        print("1..0")
        raise SystemExit

print("1..3")

import perl
import re

perl.eval("""
#line 14 "safecall"

#$^W = 1;
require Opcode;

sub compile {
    my($code) = @_;

    $code = "package main; sub do { " . $code . "}";
    #print "[[$code]]\\n";

    eval $code;
    die if $@;
}

sub foo { 42; }

*Safe1::_compile = \&compile;
""")

mask = perl.call("Opcode::opset", "bless", "add")
#print perl.call_tuple("Opcode::opset_to_ops", mask)

perl.safecall("Safe1", mask, ('_compile', 'my $n = shift; print "ok $n\\n";'))
perl.safecall("Safe1", mask, ('do', 1))

# try a trapped opcode
try:
    perl.safecall("Safe1", mask, ('_compile', 'return bless {}, "Foo"'))
except perl.PerlError as v:
    #print v
    if not re.match('^\'bless\' trapped by operation mask', str(v)): print("not ", end=' ')
    print("ok 2")

# The following call reset the perl parser state enought to
# avoid the 'nexttoke' bug.
perl.eval(""" sub ffff {}""")

perl.eval("""
sub foo {
    print "not ";
    Safe1::foo(@_);
}

sub Safe1::foo {
    my $n = shift;
    print "ok $n\\n";
}

""")


perl.safecall("Safe1", mask, ('_compile',
			      'foo(shift)'))
perl.safecall("Safe1", mask, ('do', 3))



print("1..12")

import perl
h = perl.get_ref("%")

if h.__type__ != 'HASH' or h.__class__ != None: print("not ", end=' ')
print("ok 1")

if len(h) != 0 or len(list(h.items())) != 0 or \
   len(list(h.keys())) != 0 or len(list(h.values())) != 0:
    print("not ", end=' ')
print("ok 2")

try:
    print(h[42])
except TypeError as v:
    if str(v) != 'perl hash key must be string': print("not ", end=' ')
    print("ok 3")

try:
    print(h["foo"])
except KeyError as v:
    if str(v) != "'foo'": print("not ", end=' ')
    print("ok 4")

if h.get("foo") != None or h.get("foo", 42) != 42: print("not ", end=' ')
print("ok 5")

try:
    print(h.get(42))
except TypeError as v:
    if str(v) != "get() argument 1 must be string or read-only buffer, not int":
        print("not ", end=' ')
    print("ok 6")

h["foo"] = 42

if len(h) != 1 or h["foo"] != 42: print("not ", end=' ')
print("ok 7")

h["bar"] = 21

# here we assume a certain order, which might get broken by another hash
# algoritim or other internal changes.  In that case fix the tests below.
if list(h.keys()) != ["bar", "foo"] or \
   list(h.values()) != [21, 42] or \
   list(h.items()) !=  [("bar", 21), ("foo", 42)]:
    print("not ", end=' ')
print("ok 8")

if "baz" in h or "bar" not in h: print("not ")
print("ok 9")

h2 = h.copy()
if id(h) == id(h2) or list(h.items()) != list(h2.items()): print("not ", end=' ')
print("ok 10")

h2.clear()
if len(h2) != 0: print("not ")
print("ok 11")

del h["bar"]
if list(h.keys()) != ["foo"]: print("not ", end=' ')
print("ok 12")




#!/usr/bin/env python3

from setuptools import setup, Extension
from distutils.command.install import install
from distutils.command.build import build
import sysconfig
import os

DEBUG = 0
perl = 'perl'

import os
import sys
import subprocess
MULTI_PERL = os.path.isfile("MULTI_PERL")
BOOT_FROM_PERL = os.path.isfile("BOOT_FROM_PERL")

p = os.popen("perl -MExtUtils::Embed -e 'ccopts();print \"\n\"; ldopts()'")
perl_ccopts = p.readline().strip()
perl_ldopts = p.readline().strip()
p.close()

ext_name     = "perl"
include_dirs = []
macros       = []
cc_extra     = []
cc_extra.append(perl_ccopts)

for x in perl_ccopts.split():
    if x[:2] == '-I':
        include_dirs.append(x[2:])

    # XXX This is disabled since distutils does not yet implement
    # define_macros.  Aarghhh!!  So much time wasted on debugging
    # because of this.
    elif 0 and x[:2] == '-D':
        m = x[2:].split('=', 2)
        if len(m) == 1:
            m.append(None)
        macros.append(tuple(m))
    else:
        cc_extra.append(x)

lib_dirs  = []
libs      = []
ld_extra  = []
o_extra   = []
sym_extra   = []

extra_ext = []

ld_extra.extend(perl_ldopts.split())

# Hack name to get it to compile as C++ file on Windows
svrv_object_c_name = "svrv_object.c"
if sys.platform[:3] == "win":
    import shutil
    svrv_object_c_name = "svrv_object.cpp"
    if os.path.exists(svrv_object_c_name):
        os.chmod(svrv_object_c_name, 777)
        os.unlink(svrv_object_c_name)
    shutil.copy("svrv_object.c", svrv_object_c_name)

sources = ['perlmodule.c',
        'lang_lock.c',
        'lang_map.c',
        svrv_object_c_name,
        'pyo.c',
        'try_perlapi.c',
        ]

if BOOT_FROM_PERL:
    cc_extra.append("-DBOOT_FROM_PERL")
else:
    for x in perl_ldopts.split():
        if x[:2] == '-L':
            lib_dirs.append(x[2:])
        elif x[:2] == '-l' and sys.platform != 'win32':
            libs.append(x[2:])
        elif x[:1] != '-' and (x[-3:] == '.so' or
                x[-2:] == '.o'  or
                x[-2:] == '.a'
                ):
            o_extra.append(x)
        else:
            ld_extra.append(x)

    if not os.path.isfile("perlxsi.c"):
        os.system(perl + " -MExtUtils::Embed -e xsinit")
    sources.append('perlxsi.c');

    # Try to figure out if we use dlopen on this platform
    p = os.popen(perl + ' -V:dlsrc')
    dlsrc = p.readline()
    p.close()
    if dlsrc == "dlsrc='dl_dlopen.xs';\n":
        ext_name = "perl2"
        cc_extra.append('-DDL_HACK')
        cc_extra.append('-DEXT_SUFFIX="' + sysconfig.get_config_vars().get("EXT_SUFFIX", ".so") + '"')
        extra_ext.append(Extension(name = "perl",
            extra_compile_args = cc_extra,
            sources = ["dlhack.c"],
            libraries= ["dl"],
            ))


if MULTI_PERL:
    cc_extra.append("-DMULTI_PERL")
    sources.append('thrd_ctx.c')

if not os.path.isfile("try_perlapi.c") or \
        os.path.getmtime("try_perlapi.c") < os.path.getmtime("try_perlapi.pl"):
            os.system(perl + " try_perlapi.pl")

if sys.argv[1] == "test" or sys.argv[1] == "build" or sys.argv[1] == "install":
    cwd=os.getcwd()
    ldpath = '%s/Python-Object/blib/arch/auto/Python/Object' % cwd
    perllib = '%s/Python-Object/blib/lib' % cwd
    perlarchlib = '%s/Python-Object/blib/arch' % cwd
    os.environ["PERL5LIB"] = "%s:%s%s" % (perllib, perlarchlib, (":"+os.environ["PERL5LIB"] if "PERL5LIB" in os.environ else ''))
    os.environ["LD_LIBRARY_PATH"] = "%s%s" % (ldpath, (":"+os.environ["LD_LIBRARY_PATH"] if "LD_LIBRARY_PATH" in os.environ else ''))
    os.environ["PYTHON"] = sys.executable


if sys.platform == 'win32':
    libs.append('perl56')
    for x in ['15','16','20']:
        if os.access(os.path.join(sys.prefix, 'libs', 'python'+x+'.lib'), os.F_OK):
            libs.append('python'+x)
    sym_extra.append('get_thread_ctx')
    sym_extra.append('sv2pyo')
    sym_extra.append('pyo2sv')
    sym_extra.append('newPerlPyObject_noinc')
    sym_extra.append('newPerlPyObject_inc')
    sym_extra.append('PerlPyObject_pyo')
    sym_extra.append('PerlPyObject_pyo_or_null')
    sym_extra.append('vtbl_free_pyo')

if DEBUG:
    print(("Macros:", macros))
    print(("Include: ", include_dirs))
    print(("Extra CC: ", cc_extra))
    print(("Obj: ", o_extra))
    print(("Libs:", libs))
    print(("Lib dirs:",  lib_dirs))
    print(("Extra LD: ", ld_extra))

ext_modules = []
ext_modules.append(Extension(name = ext_name,
    sources = sources,
    define_macros = macros,
    include_dirs = include_dirs,
    extra_compile_args = cc_extra,
    extra_objects =  o_extra,
    libraries = libs,
    library_dirs = lib_dirs,
    extra_link_args = ld_extra,
    export_symbols = sym_extra,
    ))
ext_modules.extend(extra_ext)

class build_perl(build):
    def run(self):
        os.chdir('Python-Object')
        build.spawn(self, ['perl','Makefile.PL', 'INSTALLDIRS=vendor'], 'Python-Object')
        build.spawn(self, ['make'])
        os.chdir('..')
        build.run(self)

class my_install(install):

    def run(self):
        cur_dir = os.getcwd()
        multi_perl = os.path.join(cur_dir, 'MULTI_PERL')
        if os.access(multi_perl, os.F_OK):
            os.unlink(multi_perl)
        if self.root is None:
            self.root = ''

        os.chdir('Python-Object')
        install.spawn(self, ["make",] + (["DESTDIR=%s" % self.root] if len(self.root) else []) + ["install",])
        os.chdir('..')
        if "-DMULTI_PERL" in cc_extra:
            cc_extra.pop(cc_extra.index("-DMULTI_PERL"))
            sources.pop(sources.index('thrd_ctx.c'))
        # Run actual install
        install.run(self)

setup (name        = "pyperl",
        version     = "1.0.3",
        description = "Embed a Perl interpreter",
        url         = "http://www.ActiveState.com",
        author      = "ActiveState",
        author_email= "gisle@ActiveState.com",
        py_modules  = ['dbi', 'dbi2', 'perlpickle', 'perlmod'],
        ext_modules = ext_modules,
        cmdclass    = { 'install': my_install, 'build': build_perl},
        test_suite  = "tests"
        )

# vim:ts=4:sw=4:et



print("1..5")

import perl
perl.eval("""

sub foo;

sub bar { }

@baz = ();

$Foo::bar = 33;

""")

if perl.defined("baz") or perl.defined("baz"): print("not ", end=' ')
print("ok 1")

if not perl.defined("foo") and perl.defined("bar"): print("not ", end=' ')
print("ok 2")

if not perl.defined("@baz"): print("not ", end=' ')
print("ok 3")

if not perl.defined("$Foo::bar"): print("not ", end=' ')
print("ok 4")

try:
    if perl.defined(" $Foo::bar"): print("not ", end=' ')
except perl.PerlError:
    print("ok 5")




import perl
#if perl.MULTI_PERL:
#	print "1..0"
#	raise SystemExit

print("1..11")


perl.eval("""

sub foo {
    if (wantarray) {
	return "array";
    }
    elsif (defined wantarray) {
        return "scalar";
    }
    else {
        return;
    }
}

""")

foo = perl.eval("\&foo")

testno = 1

def expect(res, expect):
    global testno
    if res != expect:
	print("Expected", repr(expect), "got", repr(res))
	print("not", end=' ')
    print("ok", testno)
    testno = testno + 1

void   = None
scalar = "scalar"
array  = ("array",)

expect(foo(), scalar)
expect(foo(__wantarray__ = 1), array)
expect(foo(__wantarray__ = None), void)

foo.__wantarray__ = 1;
expect(foo(), array)
expect(foo(__wantarray__ = 0), scalar)

expect(foo(__wantarray__ = None), void)

foo.__wantarray__ = None
expect(foo(), void)

expect(perl.call("foo"), scalar)
expect(perl.call_tuple("foo"), array)

expect(perl.call("foo", __wantarray__ = 1), array)
expect(perl.call_tuple("foo", __wantarray__ = 0), scalar)



print("1..6")

import perl

class Foo:
    def foo(self, a, b=None):
        return "a=" + str(a) + ", b=" + str(b)

perl.eval("""

use Python qw(apply KW);

$| = 1;

sub foo {
    my $o = shift;

    eval {
        $o->foo();
    };
    #print $@;
    print "not " unless "$@" =~ /^python.<type 'exceptions.TypeError'>: foo\(\) takes at least 2 arguments \(1 given\)/;
    print "ok 1\n";

    my $res;

    # Test glob version
    $res = $o->foo(101);
    #print "$res\\n";
    print "not " unless $res eq "a=101, b=None";
    print "ok 2\n";

    {
       package Foo::Bar;
       $res = $o->foo(*b => 102, *a => 101);
       #print "$res\\n";
       print "not " unless $res eq "a=101, b=102";
       print "ok 3\n";
    }

    $res = $o->foo(101, *b => 102);
    #print "$res\\n";
    print "not " unless $res eq "a=101, b=102";
    print "ok 4\n";

    # Test KW constructor
    $kw = KW(b => 102);
    $kw->{a} = 101;

    $res = $o->foo($kw);
    #print "$res\\n";
    print "not " unless $res eq "a=101, b=102";
    print "ok 5\n";

    $res = $o->foo(KW(a => 101));
    #print "$res\\n";
    print "not " unless $res eq "a=101, b=None";
    print "ok 6\n";
}

""")

perl.call("foo", Foo())





# Copyright 2000-2001 ActiveState

"""Python DBI - Interface to Perl's DBI from Python.

A connection to the database is created and returned by the connect()
function.  Example:

  dbh = dbi.connect("DBI:mysql:database=test", "username", "password"
                     RaiseError = 1,
                     PrintError = 0,
                     AutoCommit = 1,
                    )

  sth = dbh.prepare("select * from foo")
  rows = sth.execute()
  while 1:
      row = sth.fetchrow_tuple()
      if not row:
          break
      print row

The object returned by connect() is actually the Perl DBI datahandle
object.  This means that the interface provided with be exactly like
that of the Perl DBI.  Extended documentation on DBI available from
the 'perldoc DBI' command.

The dbi2 module provide an interface to Perl DBI that conforms to
Python's own DB API.
"""

import perl

perl.require("DBI")


def connect(data_source, username, password="", **attr):
    """Make a new connection to the database

The first parameter is the data_source string (something beginning with "DBI:").
Then there is a username and a password and at last other named configuration
parameters like; RaiseError, PrintError and AutoCommit.
"""
    dbh = perl.callm("connect", "DBI", data_source, username, password,
            dict2hash(attr))
    return dbh

def available_drivers():
    return perl.callm_tuple("available_drivers", "DBI")

def data_sources(driver):
    return perl.callm_tuple("data_sources", "DBI", driver)

def trace(level, filename=None):
    return perl.callm("trace", "DBI", level, filename)

def dict2hash(dict_):
    hash_ = perl.get_ref("%")
    hash_.update(dict_)
    return hash_




print("1..5")
import perl
import sys;

perl.eval("""

my $sys = Python::Import("sys");
print "not " unless $sys->version eq "%s";
print "ok 1\n";

my $string = Python::Import("string");

print "not " unless $string->digits eq join("", 0..9);
print "ok 2\n";

print "not " unless $string->lower("ABC") eq "abc";
print "ok 3\n";

eval {
   $fail = Python::Import("not_existing");
};
#print $@;

print "not " unless Python::Err::ImportError($@) && $@ =~ /No module named/;
print "ok 4\n";

""" % (sys.version))

if sys.modules['string']:
    print("ok 5")




# Verify that perl executes concurrently as it should under MULTI_PERL

import perl
if not perl.MULTI_PERL:
        print("1..0")
        raise SystemExit

print("1..10")

import _thread
import time

def t(start, step, stop):
    perl.eval("""

    my($start, $step, $stop) = (%d,%d,%d);

    $| = 1;
    for (my $i = $start; $i <= $stop; $i += $step) {
        print "ok $i\n";
        sleep(1);
    }
    
    """ % (start, step, stop))

_thread.start_new_thread(t, (1, 2, 9))
time.sleep(0.5)
_thread.start_new_thread(t, (2, 2, 10))

perl.eval("sleep 3")
#print "perl sleep done"
time.sleep(4)
#print "done"



# Provide an interface to DBI that conforms to the Python DB API spec
# http://www.python.org/topics/database/DatabaseAPI-2.0.html
#
# Todo:
#   - get exceptions right
#   - improve efficiency
#   - give access extra DBI methods and Driver specific methods
#   - how to deal with datefields?
#
# Copyright 2001 ActiveState

"""Expose Perl's DBI though the Python DB API v2 interface.

This module give access to Perl's DBI through with an interface that
conforms to the DB API:

    http://www.python.org/topics/database/DatabaseAPI-2.0.html

This module should probably not be regarded as thread safe since there
might be Perl DBI drivers that are not.  The DBI paramstyle is
'qmark'.

Example:

    import dbi2

    d = dbi2.connect("DBI:mysql:database=test", "root", "")

    c = d.cursor()

    c.execute("select * from foo")
    print "Rowcount =", c.rowcount

    print c.description
    while 1:
        row = c.fetchone()
        if not row:
           break
        print row

    c.close()

The 'dbi' module provide an interface to the raw Perl DBI.
"""

apilevel = "2.0"  # target, but not really there yet
threadsafety = 0
paramstyle="qmark"

perl = None

def connect(*arg):
    """Connect to the database

The first parameter is the DBI source string.  Then optional username
and password follows.
"""
    return connection(*arg)

class connection:
    def __init__(self, dsn, user=None, password=None):
        global perl
        if not perl:
            import perl
            perl.require("DBI")

        conf = perl.get_ref("%")
        conf["RaiseError"] = 0
        conf["PrintError"] = 0
        conf["AutoCommit"] = 1

        self.dbh = perl.callm("connect", "DBI", dsn, user, password, conf)
        if self.dbh is None:
            raise OperationalError(perl.eval("$DBI::errstr"))

        self.dbh["RaiseError"] = 1
        
        try:
            self.dbh["AutoCommit"] = 0
            self.autocommit = 0
        except:
            self.autocommit = 1
            

    def commit(self):
        self.dbh.commit()

    def rollback(self):
        self.dbh.rollback()

    def cursor(self):
        return cursor(self.dbh)

    def close(self):
        self.dbh.disconnect()
        self.dbh = None

    def __del__(self):
        try:
            # XXX should we??
            self.dbh.disconnect()
        except:
            pass

class cursor:
    def __init__(self, dbh):
        self.dbh = dbh
        self.arraysize = 1

    def execute(self, operation, *param):
        self.sth = self.dbh.prepare(operation)
        self.sth.execute(*param)
        self._get_types()

    def executemany(self, operation, parameters):
        self.sth = self.dbh.prepare(operation)
        for p in parameters:
            self.sth.execute(*p)
        self._get_types()
        return rowcount

    def _get_types(self):
        self.types = [TYPES.get(s, (s, str)) for s in tuple(self.sth["TYPE"])]

    def __getattr__(self, attr):
        if attr == "rowcount":
            return self.sth.rows()
        elif attr == "description":
            return self._description()
        else:
            raise AttributeError(attr)
        
    def _description(self):
        name = tuple(self.sth["NAME"])
        dsize = (0,) * len(name) # DBI does have this info
        isize = dsize
        types = [s[0] for s in self.types]
        prec = tuple(self.sth["PRECISION"])
        scale = tuple(self.sth["SCALE"])
        null = tuple(self.sth["NULLABLE"])
        return list(zip(name, dsize, isize, types, prec, scale, null))

    def fetchone(self):
        row = list(self.sth.fetchrow_tuple())
        if not row:
            return None
        for i in range(len(self.types)):
            row[i] = self.types[i][1](row[i])
        return row

    def fetchmany(self, size=None):
        if not size:
            size = self.arraysize
        res = []
        while size >= 1:
            row = self.fetchone()
            if not row:
                break
            res.append(row)
            size = size - 1
        return res

    def fetchall(self):
        res = []
        while 1:
            row = self.fetchone()
            if not row:
                break
            res.append(row)
        return res

    def setinputsizes(self, sizes):
        pass

    def setoutputsize(self, size, column=None):
        pass

    def close(self):
        try:
            self.sth.finish()
        except:
            pass
        self.sth = None
        self.dbh = None

# Type constructors
def Date(year, month, day):
    return "%04d-%02d-%02d" % (year, month, day)

def Time(hour, min, sec):
    return "%2d:%02d:%02d" % (hour, min, sec)

def Timestamp(year, month, day, hour, min, sec):
    return Date(year, month, day) + " " + Time(hour, min, sec)

import time

def DateFromTicks(ticks):
    return Date(*time.localtime(ticks)[:3])

def TimeFromTicks(ticks):
    return Time(*time.localtime(ticks)[3:6])

def TimestampFromTicks(ticks):
    return Timestamp(*time.localtime(ticks)[:6])

Binary = str

def toNum(s):
    float(s)

TYPES = {
      1: ("CHAR", str),
      2: ("NUMERIC", toNum),
      3: ("DECIMAL", toNum),
      4: ("INTEGER", int),
      5: ("SMALLINT", int),
      6: ("FLOAT", float),
      7: ("REAL", float),
      8: ("DOUBLE", float),
      9: ("DATE", str),
     10: ("TIME", str),
     11: ("TIMESTAMP", str),
     12: ("VARCHAR", str),
     -1: ("LONGVARCHAR", str),
     -2: ("BINARY", str),
     -3: ("VARBINARY", str),
     -4: ("LONGVARBINARY", str),
     -5: ("BIGINT", int),
     -6: ("TINYINT", int),
     -7: ("BIT", int),
     -8: ("WCHAR", str),
     -9: ("WVARCHAR", str),
    -10: ("WLONGVARCHAR", str),
}

class TYPE:
    def __init__(self,*values):
        self.values = values

    def __cmp__(self,other):
        if other in self.values:
            return 0
        if other < self.values:
            return 1
        else:
            return -1

STRING   = TYPE("CHAR", "VARCHAR", "LONGVARCHAR")
BINARY   = TYPE("BINARY", "VARBINARY", "LONGVARBINARY")
NUMBER   = TYPE("NUMERIC", "DECIMAL", "INTEGER", "SMALLINT",
                "FLOAT", "REAL", "DOUBLE",
                "BIGINT", "TINYINT", "BIT")
DATETIME = TYPE("DATE", "TIME", "TIMESTAMP")
ROWID    = TYPE()

## /* Standard SQL datatypes (ANSI/ODBC type numbering)            */
## #define SQL_ALL_TYPES           0
## #define SQL_CHAR                1
## #define SQL_NUMERIC             2
## #define SQL_DECIMAL             3
## #define SQL_INTEGER             4
## #define SQL_SMALLINT            5
## #define SQL_FLOAT               6
## #define SQL_REAL                7
## #define SQL_DOUBLE              8
## #define SQL_DATE                9       /* SQL_DATETIME in CLI! */
## #define SQL_TIME                10
## #define SQL_TIMESTAMP           11
## #define SQL_VARCHAR             12

## /* Other SQL datatypes (ODBC type numbering)                    */
## #define SQL_LONGVARCHAR         (-1)
## #define SQL_BINARY              (-2)
## #define SQL_VARBINARY           (-3)
## #define SQL_LONGVARBINARY       (-4)
## #define SQL_BIGINT              (-5)    /* too big for IV       */
## #define SQL_TINYINT             (-6)

## /* Support for Unicode and SQL92 */
## #define SQL_BIT                 (-7)
## #define SQL_WCHAR               (-8)
## #define SQL_WVARCHAR            (-9)
## #define SQL_WLONGVARCHAR        (-10)


# Exceptions
import exceptions

class Error(exceptions.Exception):
    pass

class Warning(exceptions.Exception):
    pass

class InterfaceError(Error):
    pass

class DatabaseError(Error):
    pass

class InternalError(DatabaseError):
    pass

class OperationalError(DatabaseError):
    pass

class ProgrammingError(DatabaseError):
    pass

class IntegrityError(DatabaseError):
    pass

class DataError(DatabaseError):
    pass

class NotSupportedError(DatabaseError):
    pass

if "zip" not in __builtins__:
    def zip(*args):
        # for 1.5.2 which does not provide zip as a builtin
        i = 0
        res = []
        try:
            while 1:
                e = []
                for a in args:
                    e.append(a[i])
                res.append(tuple(e))
                i = i + 1
        except IndexError:
            pass
        return tuple(res)



#!/usr/bin/env python

import unittest
import perl

class TestPyPerlHashOp(unittest.TestCase):

    def setUp(self):
        self.h = perl.get_ref("%")

    def test_type_is_hash(self):
        self.assertEqual(self.h.__type__, 'HASH')
        self.assertEqual(self.h.__class__, None)

    def test_hash_elements(self):
        self.assertEqual(len(self.h), 0)
        self.assertEqual(len(list(self.h.items())), 0)
        self.assertEqual(len(list(self.h.keys())), 0)
        self.assertEqual(len(list(self.h.values())), 0)


    def test_hash_type_error(self):
        try:
            print((self.h[42]))
        except TypeError as v:
            self.assertEqual(str(v), 'perl hash key must be string')

    def test_hash_key_error(self):
        try:
            print((self.h["foo"]))
        except KeyError as v:
            self.assertEqual(str(v), "'foo'")

    def test_hash_get(self):
        self.h["foo"] = 42
        self.assertIsNotNone(self.h.get("foo"))
        self.assertEqual(self.h.get("foo"), 42)

    def test_hash_get_type_error(self):
        try:
            print((self.h.get(42)))
        except TypeError as v:
            self.assertEqual(str(v), "a bytes-like object is required, not 'int'")

    def test_hash_key_index(self):
        self.h["foo"] = 42
        self.assertEqual(len(self.h), 1)
        self.assertEqual(self.h["foo"], 42)

    def test_hash_algoritm_order(self):
        self.skipTest("Order doesn't to predictable...")
        self.h["foo"] = 42
        self.h["bar"] = 21

        # self.here we assume a certain order, which might get broken by another self.hash
        # algoritim or other internal changes.  In that case fix the tests below.
        self.assertEqual(list(self.h.keys()), ["bar", "foo"])
        self.assertEqual(list(self.h.values()), [21, 42])
        self.assertEqual(list(self.h.items()), [("bar", 21), ("foo", 42)])

    def test_hash_has_key(self):
        self.h["bar"] = 21

        self.assertFalse("baz" in self.h)
        self.assertTrue("bar" in self.h)

    def test_hash_copy(self):
        self.h2 = self.h.copy()
        self.assertNotEqual(id(self.h), id(self.h2))
        self.assertEqual(list(self.h.items()), list(self.h2.items()))

    def test_hash_clear(self):
        self.h2 = self.h.copy()
        self.h2.clear()
        self.assertEqual(len(self.h2), 0)

    def test_hash_delete(self):
        self.h["foo"] = 42
        self.h["bar"] = 21
        del self.h["bar"]
        self.assertEqual(list(self.h.keys()), ["foo"])

def test_main():
    from test import test_support
    test_support.run_unittest(TestPyPerlHashOp)

if __name__ == "__main__":
    test_main()
